#!/usr/bin/perl -w
#
#	clamstats.pl v1.5
#	10.25.04 Paul Venezia
#

use strict;
no warnings "uninitialized";
use CGI qw/:standard *table start_ul/;

my $CLAMAV_ROOT = "/var/log/clamav/";
# my $CLAMAV_ROOT = "./";
# my $CLAMAV_ROOT = "./";
my $logfile = $CLAMAV_ROOT . "clamd.log";
my $fclogfile = $CLAMAV_ROOT . "freshclam.log";
my $host = `hostname`;

my $text = "1";
my $html = "";
if (defined $ARGV[0] && $ARGV[0] =~ /--html/) {
	$html = "true";
	undef $text;
}

my %stats = ();
$stats{count} = 0;
my ($clamdate, $startdate, $TXT, $lastmon, $fcdate, $fcmainversion, $fcmaincount, $fcdailyversion, $fcdailycount, $fcsigcount, $fcerror, $fcerrors, $fcerrordate, $dbstatus);
$startdate = "06 January 2009";
my $hourmax = 0;
my $datemax = 0;
my $monthmax = 0;
my $yearmax = 0;
my $total = 0;
my ($hourscale, $datescale, $monthscale, $yearscale);

my %months = (
    'Jan'  =>  1,
    'Feb'  =>  2,
    'Mar'  =>  3,
    'Apr'  =>  4,
    'May'  =>  5,
    'Jun'  =>  6,
    'Jul'  =>  7,
    'Aug'  =>  8,
    'Sep'  =>  9,
    'Oct'  => 10,
    'Nov'  => 11,
    'Dec'  => 12,
);

sub build_key {
    my ($month, $day) = (/^(\w+)\s+(\d{2})$/);
    return(sprintf "%-2.2d%-2.2d", $months{$month}, $day);
}

sub build_key_mon {
    return(sprintf "%-2.2d", $months{$_});
}


sub swrite {
  my $fmt = shift(@_);
  $^A = '';
  formline($fmt,@_);
  return $^A;
}

sub read_freshclam_log () {
	open (FCLOG, "<$fclogfile") or die "Couldn't read freshclam logfile $fclogfile\n";
	foreach (<FCLOG>) {
	        s/\s+$//;
		#freshclam[9848]: ClamAV update process started at Fri Mar  2 11:04:00 2007
		if (/ClamAV update process started at (... ... .. ..:..:.. \d{4})$/) {
			$fcdate = $1;
		}
#		if (/main\.cld.+\(version:\s(\d+),\ssigs:\s(\d+),/) {
		if (/main\.(?:cvd|cld).+\(version:\s(\d+),\ssigs:\s(\d+),/) {
			#print "MAIN: $2\n";
			$fcmainversion = $1;
			$fcmaincount = $2;
		}
		if (/daily\.(?:cvd|cld).+\(version:\s(\d+),\ssigs:\s(\d+),/) {
			#print "DAILY: $2\n";
			$fcdailyversion = $1;
			$fcdailycount = $2;
		}
		if (/^ERROR:\s(.*)/) {
			$fcerrors++;
			$fcerror = $1;
			$fcerrordate = $fcdate;
		}
	}
	$fcsigcount = ($fcmaincount + $fcdailycount);
	1 while $fcsigcount =~ s/(.*\d)(\d\d\d)/$1,$2/;
}

&read_freshclam_log;

open (LOG, "<$logfile") or die "Couldn't read $logfile.\n";

sub addstats($ $ $) {
    my ($date,$virus,$ext) = @_;
    my ($dow,$month,$day,$time,$year) = ($date =~ /^(...) (...) +(\d+) (\d+:\d+:\d+) (\d+)$/ );

    #print "ADDSTATS: DATE: $date, VIRUS: $virus, EXT: $ext\n";

    $day = sprintf ("%02d", $day);
    $date = "$dow $month $day $time $year";
    $startdate = $date	 if (!defined $startdate);
    my ($hour, $min, $sec) = split(":", $time);

    $date =~ s/\d+:\d+:\d+\s//;	
    $date =~ s/^\w+\s//;	
    $date =~ s/\s\d+$//;	

    $stats{date}{$date}++;
    $stats{hour}{$hour}++;
    $stats{month}{$month}++;
    $stats{year}{$year}++;
    $stats{virus}{$virus}++;
    $stats{count}++;
    $stats{type}{$ext}++;

    $hourmax = $stats{hour}->{$hour} if ($stats{hour}->{$hour} > $hourmax);
    $datemax = $stats{date}->{$date} if ($stats{date}->{$date} > $datemax);
    $monthmax = $stats{month}->{$month} if ($stats{month}->{$month} > $monthmax);
    $yearmax = $stats{year}->{$year} if ($stats{year}->{$year} > $yearmax);
}

foreach (<LOG>) {

    s/\s+$//;

    #--- Stopped at Fri Jul 15 11:50:27 2005
    #+++ Started at Fri Jul 15 11:50:28 2005

    $clamdate = $1 if /[+]{3} Started at (.+)$/;
    if (!defined $clamdate) {
	$clamdate = ""; 
    }

    # /tmp/somefile: HTML.Phishing.Bank-1 FOUND
    my ($id, $ext, $virus);

    if (my ($date,$log) = split / -> /) {
	#print "SPLIT: DATE: \"$date\", LOG: $log\n";

	# Sun Jun 11 12:15:28 2006 -> /var/spool/MIMEDefang/run/mdefang-k5BGFSK0011658/./Work/msg-18091-564.txt: OK
	# Mon Jan 19 11:41:47 2004 -> /var/spool/MIMEDefang/run/mdefang-i0JGfhb5024243/./Work/msg-9414-65.exe: Worm.Gibe.F FOUND

	if ( $date =~ /... ... +\d+ \d+:\d+:\d+ \d+/ ) {

	    if (($id,$ext,$virus) = ($log =~ m{mdefang-(\w+).*/Work/msg[-\d]\.(\w+):\s+(.+) FOUND} )) {
		print "DEFANG: ID: $id, EXT: $ext, VIRUS: $virus\n";
		addstats($date,$virus,$ext);

#	    } elsif (($virus) = ($log =~ /^stream(?: \d+)?: (.+) FOUND/ )) {
            } elsif (($virus) = ($log =~ /^stream[^:]+: (.+) FOUND/ )) {
		# Tue Feb 13 23:03:04 2007 -> stream: Html.Phishing.Bank.Gen175.Sanesecurity.06032008 FOUND
		# Fri Mar  2 21:00:00 2007 -> stream 1081: Eicar-Test-Signature FOUND
		$id =  'N/A';
		#print "STREAM: ID: $id, EXT: $ext, VIRUS: $virus\n";
		addstats($date,$virus,'N/A');
	    }
	}
    }

    #Mar  2 18:04:00 myhost clamd[18111]: SelfCheck: Database status OK. 
    #Tue Feb  3 05:09:31 2004 -> /tmp/MzuO9oRPFN: Worm.SCO.A FOUND
    #Tue Feb  3 05:18:05 2004 -> Session 2 stopped due to timeout.
    #Tue Feb  3 07:09:34 2004 -> SelfCheck: Database status OK.

    elsif (/(?:(.+) -> )?SelfCheck: Database status OK/) {
	$dbstatus = $1;
    }
    close (LOG);
}

$total = $stats{count};
1 while $total =~ s/(.*\d)(\d\d\d)/$1,$2/;

#$hourscale = int($hourmax/200);
#$hourscale = 1 if $hourscale == 0;
#$datescale = int($datemax/200);
#$datescale = 1 if $datescale == 0;
#$monthscale = int($monthmax/160);
#$monthscale = 1 if $monthscale == 0;
#$yearscale = int($yearmax/140);
#$yearscale = 1 if $yearscale == 0;

&text_out if $text;
&html_out if $html;

sub text_out () {

	$hourscale = int($hourmax/50);
	$hourscale = 1 if $hourscale == 0;
	$datescale = int($datemax/50);
	$datescale = 1 if $datescale == 0;
	$monthscale = int($monthmax/30);
	$monthscale = 1 if $monthscale == 0;
	$yearscale = int($yearmax/20);
	$yearscale = 1 if $yearscale == 0;

	
	$TXT = "\nClamAV Statistics\n$host\n";
	$TXT .= "--------------------------------------------------------\n";
	$TXT .= "clamd last started\t\t$clamdate\n";
	$TXT .= "--------------------------------------------------------\n";
	$TXT .= "Statistics since\t\t$startdate\n";
	$TXT .= "Last Database Update\t\t$fcdate\n";
	$TXT .= "--------------------------------------------------------\n";
	$TXT .= "Total viruses detected\t\t$total\n";
	$TXT .= "Total Database Signatures\t$fcsigcount\n";
	if ($fcerror) {
		$TXT .= "--------------------------------------------------------\n";
		$TXT .= "$fcerrors FreshClam errors, last on $fcerrordate: $fcerror\n";
	}
	$TXT .= "\n";
	
	&process_output;

	$TXT .= "\n";
	print $TXT;

}

sub html_out () {

	$hourscale = int($hourmax/200);
	$hourscale = 1 if $hourscale == 0;
	$datescale = int($datemax/150);
	$datescale = 1 if $datescale == 0;
	$monthscale = int($monthmax/160);
	$monthscale = 1 if $monthscale == 0;
	$yearscale = int($yearmax/140);
	$yearscale = 1 if $yearscale == 0;

	print "<HTML><HEAD><TITLE>ClamAV - Clamd Statistics for $host</TITLE>";
	print "<META HTTP-EQUIV=\"PRAGMA\" CONTENT=\"no-cache\">";
	print "<BODY BGCOLOR=WHITE><CENTER>";

	print "<table border=0 cellpadding=3><tr><Th COLSPAN=3><font size=5>ClamAV Statistics<br>$host</font></th><TH align=center><tr>";
	print "<TD ALIGN=LEFT bgcolor=\#9999FF>Clamd last started</TD><TD ALIGN=RIGHT bgcolor=\#DCDCEE>$clamdate</TD><TR>";
	print "<TD ALIGN=LEFT bgcolor=\#9999FF>Statistics since</TD><TD ALIGN=RIGHT bgcolor=\#DCDCEE>$startdate</TD><TR>";
	print "<TD ALIGN=LEFT bgcolor=\#9999FF>Last Database Update</TD><TD ALIGN=RIGHT bgcolor=\#DCDCEE>$fcdate</TD><TR>";
	print "<TD ALIGN=LEFT bgcolor=\#9999FF>Database Status OK</TD><TD ALIGN=RIGHT bgcolor=\#DCDCEE>$dbstatus</TD><TR>";
	print "<TD ALIGN=LEFT bgcolor=\#9999FF>Database Signatures</TD><TD ALIGN=RIGHT bgcolor=\#DCDCEE>$fcsigcount</TD><TR>";
	print "<TD ALIGN=LEFT bgcolor=\#9999FF>Total Viruses Detected</TD><TD ALIGN=RIGHT bgcolor=\#DCDCEE>$total</TD><TR>";
	if ($fcerror) {
		print "</TABLE><br><br>";
		print "<table border=0 cellpadding=3>";
		print "<TD ALIGN=LEFT bgcolor=\#FF0000>$fcerrors FreshClam Errors, last on $fcerrordate</TD><TD ALIGN=RIGHT bgcolor=\#DCDCEE>$fcerror</TD><TR>" if $fcerror;
	}
	print "</TABLE><br><br>";
	print "<table border=0 align=center><td align=center valign=top>";
	&process_output;

	print "</TABLE>";
	&html_footer;

}

sub html_footer () {
	print "</TD><tr></table><br><br>";
	print "<br><font size=1>Generated by clamstats.pl on " . localtime() . "</font><br>";
	print "</BODY></HTML>";
}

sub process_output () {

### VIRUS COUNTS

	$TXT .= keys ( %{$stats{virus}} ) . " Virus Types Detected\n" if $text;
	$TXT .= "------------------------------------------\n" if $text;
	
	if ($html) {
		print "<table border=0><tr><Th COLSPAN=4 bgcolor=\#666699><font color=\#FFFFFF size=4>" . keys ( %{$stats{virus}} ) . " Virus Types Detected</font></th><TH align=center>";
	}

	foreach my $rec ( sort {$stats{virus}{$b}<=>$stats{virus}{$a}}  keys %{$stats{virus}} ) {
		my $percent = sprintf ("%.2f", (($stats{virus}->{$rec} / $stats{count}) * 100));
		1 while $stats{virus}->{$rec} =~ s/(.*\d)(\d\d\d)/$1,$2/;
		$TXT .= swrite("@<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<  @>>>>>>>>> @>>>>>",$rec, $stats{virus}->{$rec}, $percent)."%\n" if $text;

		if ($html) {
		        print "<tr><td align=left bgcolor=\#9999FF>";
		        print "$rec</td>";
			print "<td align=right bgcolor=\#DCDCEE>";
		        print $stats{virus}->{$rec} . "</td><td></td><td ALIGN=RIGHT bgcolor=\#CCCCFF>$percent". "%";
		        print "</td></tr>\n";
		}

	}
		print "</TABLE></td>" if $html;

### EXTENSION COUNTS
	
	$TXT .= "\n\n" . keys ( %{$stats{type}} ) . " File Extensions Used\n" if $text;
	$TXT .= "--------------------------\n" if $text;
	
	if ($html) {
		print "<td align=center valign=top><table border=0><tr><Th bgcolor=\#666699 COLSPAN=4><font color=\#FFFFFF size=4>" . keys ( %{$stats{type}} ) . " File Extensions</font></th><TH align=center>";
	}

	foreach my $typ ( sort {$stats{type}{$b}<=>$stats{type}{$a}}  keys %{$stats{type}} ) {
		my $percent = sprintf ("%.2f", (($stats{type}->{$typ} / $stats{count}) * 100));
		1 while $stats{type}->{$typ} =~ s/(.*\d)(\d\d\d)/$1,$2/;
		$TXT .= swrite("@<<<<<<<  @>>>>>> @>>>>>",$typ, $stats{type}->{$typ}, $percent)."%\n" if $text;
		
		if ($html) {
		        print "<tr><td align=left bgcolor=\#9999FF>";
		        print "$typ</td>";
		        print "<td align=right bgcolor=\#DCDCEE>";
		        print $stats{type}->{$typ} . "</td><td></td><td ALIGN=RIGHT bgcolor=\#CCCCFF>$percent" . "%";
		        print "</td></tr>\n";
		}
	}
	print "</TABLE></TD>" if $html;

### BY DATE
	
	$TXT .= "\n\nBy Date ( . = $datescale viruses )\n" if $text;
	$TXT .= "--------------------------\n" if $text;
	
	if ($html) {
                print "<td align=center valign=top><table border=0><tr><Th bgcolor=\#666699 COLSPAN=3><font color=\#FFFFFF size=4>Viruses by Date</font></th><TH align=center>";       
	}

	foreach my $dy ( map { $_->[0] }
	                 sort { $a->[1] cmp $b->[1] }
	                 map { [$_, build_key($_)] }
			 keys %{$stats{date}} ) {
	
		my $dayraw = $stats{date}->{$dy};
		1 while $stats{date}->{$dy} =~ s/(.*\d)(\d\d\d)/$1,$2/;
		$TXT .= sprintf("%6s: %6s %s\n", $dy, $stats{date}->{$dy}, "." x int($dayraw/$datescale)) if $text;

                if ($html) {
                        print "<tr><td align=left bgcolor=\#9999FF>";
                        print "$dy" . "</td>";
                        print "<td align=center bgcolor=\#DCDCEE>";
                        print $stats{date}->{$dy} . "</td>";
                        print "<td align=left bgcolor=\#DCDCEE><img src=\"/meter.jpg\" height=5  WIDTH=" . int($dayraw/$datescale) . "> ";
                        print "</td></tr>\n";
                }

	}
        print "</TABLE></TD>" if $html;

### BY HOUR
	
	$TXT .= "\n\nBy Hour ( . = $hourscale viruses )\n" if $text;
	$TXT .= "--------------------------\n" if $text;

	if ($html) {
                print "<td align=center valign=top><table border=0><tr><Th bgcolor=\#666699 COLSPAN=3><font color=\#FFFFFF size=4>Viruses by Hour</font></th><TH align=center>";       
	}

	foreach my $hr ( sort keys %{$stats{hour}} ) {
		my $hourraw = $stats{hour}->{$hr};
		1 while $stats{hour}->{$hr} =~ s/(.*\d)(\d\d\d)/$1,$2/;
		$TXT .= sprintf("%2d: %6s %s\n", $hr, $stats{hour}->{$hr}, "." x int($hourraw/$hourscale)) if $text;

		if ($html) {
                        print "<tr><td align=left bgcolor=\#9999FF>";
                        print "$hr" . "00</td>";
                        print "<td align=center bgcolor=\#DCDCEE>";
                        print $stats{hour}->{$hr} . "</td>";
			print "<td align=left bgcolor=\#DCDCEE><img src=\"/meter.jpg\" height=5  WIDTH=" . int($hourraw/$hourscale) . ">";
                        print "</td></tr>\n";
                }

	}
	print "</TABLE><BR><BR>" if $html;

### BY MONTH
	
	$TXT .= "\n\nBy Month ( . = $monthscale viruses )\n" if $text;
	$TXT .= "-------------------------\n" if $text;

        if ($html) {
                print "<table border=0><tr><Th bgcolor=\#666699 COLSPAN=3><font color=\#FFFFFF size=4>Viruses by Month</font></th><TH align=center>";
        }

	foreach my $mn ( map { $_->[0] }
	                 sort { $a->[1] cmp $b->[1] }
	                 map { [$_, build_key_mon($_)] }
			 keys %{$stats{month}} ) {
	
		$mn =~ s/^\s+//;
		my $monraw = $stats{month}->{$mn};
		1 while $stats{month}->{$mn} =~ s/(.*\d)(\d\d\d)/$1,$2/;
		$TXT .= sprintf("%6s: %6s %s\n", $mn, $stats{month}->{$mn}, "." x int($monraw/$monthscale)) if $text;

                if ($html) {
                        print "<tr><td align=left bgcolor=\#9999FF>";
                        print "$mn" . "</td>";
                        print "<td align=center bgcolor=\#DCDCEE>";
                        print $stats{month}->{$mn} . "</td>";
                        print "<td align=left bgcolor=\#DCDCEE><img src=\"/meter.jpg\" height=5  WIDTH=" . int($monraw/$monthscale) . "> ";
                        print "</td></tr>\n";
                }

	}
	print "</TABLE><br><br>" if $html;

### BY YEAR
	
	$TXT .= "\n\nBy Year ( . = $yearscale viruses )\n" if $text;
	$TXT .= "--------------------------\n" if $text;

        if ($html) {
                print "<table border=0><tr><Th bgcolor=\#666699 COLSPAN=3><font color=\#FFFFFF size=4>Viruses by Year</font></th><TH align=center>";
        }

	foreach my $yr ( sort keys %{$stats{year}} ) {
		my $yearraw = $stats{year}->{$yr};
		1 while $stats{year}->{$yr} =~ s/(.*\d)(\d\d\d)/$1,$2/;
		$TXT .= sprintf("%6s: %6s %s\n", $yr, $stats{year}->{$yr}, "." x int($yearraw/$yearscale)) if $text;
	
                if ($html) {
                        print "<tr><td align=left bgcolor=\#9999FF>";
                        print "$yr" . "</td>";
                        print "<td align=center bgcolor=\#DCDCEE>";
                        print $stats{year}->{$yr} . "</td>";
                        print "<td align=left bgcolor=\#DCDCEE><img src=\"/meter.jpg\" height=5  WIDTH=" . int($yearraw/$yearscale) . ">";
                        print "</td></tr>\n";
                }

	}
	print "</TD>" if $html;
	
	
}
